'use strict';

const assert = require('assert');
const _ = require('lodash');
const Payment = require('./wxpayv3');
const { readFileSync } = require('fs');
const { BusinessError, ErrorCode } = require('naf-core').Error;
const { AxiosService } = require('naf-framework-mongoose/lib/service');
class WeixinAuthService extends AxiosService {
  constructor(ctx) {
    super(ctx, {}, _.get(ctx.app.config, 'wxapi'));
  }

  // 通过认证码获得用户信息
  async fetch(code) {
    // TODO:参数检查和默认参数处理
    assert(code);
    const res = await this.httpGet('/api/fetch', { code });
    if (res.errcode && res.errcode !== 0) {
      this.ctx.logger.error(`[WeixinAuthService] fetch open by code fail, errcode: ${res.errcode}, errmsg: ${res.errmsg}`);
      throw new BusinessError(ErrorCode.SERVICE_FAULT, '获得微信认证信息失败');
    }
    return res;
  }
  // 获取access_token
  async gettoken() {
    const { wxapi } = this.app.config;
    const to_uri = `${wxapi.tokenUrl}?appid=${wxapi.appid}&secret=${wxapi.appsecret}&grant_type=client_credential`;
    const { access_token } = await this.httpGet(to_uri);
    return access_token;
  }
  // 下发模板消息
  async pushMould({ openid, out_trade_no }) {
    await this.userMould({ openid, out_trade_no });
    await this.adminMould({ out_trade_no });
  }
  // 管理员消息下发
  async adminMould({ out_trade_no }) {
    const { wxapi } = this.app.config;
    const access_token = await this.gettoken();
    const to_uri = `${wxapi.mould}?access_token=${access_token}`;
    const { Order: model } = this.ctx.model;
    const { Admin: adminModel } = this.ctx.model;
    const order = await model.findOne({ out_trade_no });
    const data = {
      first: {
        value: '您有新的订单',
        color: '#173177',
      },
      keyword1: {
        value: order.name,
        color: '#173177',
      },
      keyword2: {
        value: order.phone,
        color: '#173177',
      },
      keyword3: {
        value: order.hospitalName,
        color: '#173177',
      },
      keyword4: {
        value: order.subjectName,
        color: '#173177',
      },
      keyword5: {
        value: order.time,
        color: '#173177',
      },
      remark: {
        value: order.remark,
        color: '#173177',
      },
    };
    const userList = await adminModel.find();
    return Promise.all(
      userList.filter(async e => {
        if (e.openid && e.openid !== null) {
          await this.ctx.curl(to_uri, {
            method: 'POST',
            dataType: 'json',
            data: JSON.stringify({
              touser: e.openid,
              template_id: wxapi.adminTemplateId,
              data,
            }),
          });
        }
      })
    );
  }
  // 用户消息下发
  async userMould({ openid, out_trade_no }) {
    const { wxapi } = this.app.config;
    const access_token = await this.gettoken();
    const { Order: model } = this.ctx.model;
    const order = await model.findOne({ out_trade_no });
    if (access_token && order) {
      const to_uri = `${wxapi.mould}?access_token=${access_token}`;
      const data = {
        touser: openid,
        template_id: wxapi.userTemplateId,
        url: '',
        topcolor: '#FF0000',
        data: {
          first: {
            value: '感谢您登陆佳泰医疗健康管理',
            color: '#173177',
          },
          keyword1: {
            value: order.subjectName,
            color: '#173177',
          },
          keyword2: {
            value: order.hospitalName,
            color: '#173177',
          },
          keyword3: {
            value: order.name,
            color: '#173177',
          },
          keyword4: {
            value: order.phone,
            color: '#173177',
          },
          keyword5: {
            value: order.time,
            color: '#173177',
          },
          remark: {
            value: '请保持电话畅通，等待客服与您联系。',
            color: '#173177',
          },
        },
      };
      await this.ctx.curl(to_uri, {
        method: 'POST',
        dataType: 'json',
        data: JSON.stringify(data),
      });
    }
  }
  // 预支付交易单 # description = 商品描述, out_trade_no = 自定义订单号
  async orderPay({ openid, description, out_trade_no, money, _id }) {
    const { wxapi, authUrl = this.ctx.path } = this.app.config;
    const paymnet = new Payment({
      appid: wxapi.appid,
      mchid: wxapi.mchid,
      private_key: readFileSync(wxapi.wxKey).toString(), // 或者直接复制证书文件内容
      serial_no: wxapi.certid,
      apiv3_private_key: wxapi.v3key,
    });
    const host = this.ctx.header.referer.split('/')[2];
    const backUrl = encodeURI(`${this.ctx.protocol}://${host}${authUrl}`);
    // jsapi 支付下单
    const result = await paymnet.jsapi({
      description,
      out_trade_no,
      notify_url: backUrl,
      amount: {
        total: money * 100,
        currency: 'CNY',
      },
      payer: {
        openid,
      },
    });
    // 如果请求成功 redis存数据
    if (result.status === 200) {
      const { prepay_id } = JSON.parse(result.data);
      const order = JSON.stringify({ openid, prepay_id, _id, out_trade_no });
      await this.app.redis.set('orderPay', order, 'EX', 1200000);
      return false;
    }
  }
  // 跳转发起支付页面
  async pay() {
    const { wxapi } = this.app.config;
    const orderPay = await this.app.redis.get('orderPay');
    const { openid, prepay_id, out_trade_no } = JSON.parse(orderPay);
    const appid = wxapi.appid;
    // 商户密钥证书
    const privateKey = readFileSync(wxapi.wxKey).toString();
    // 32位随机字符串
    const payment = new Payment({
      appid: wxapi.appid,
      mchid: wxapi.mchid,
      private_key: readFileSync(wxapi.wxKey).toString(), // 或者直接复制证书文件内容
      serial_no: wxapi.certid,
      apiv3_private_key: wxapi.v3key,
    });
    const payNonceStr = payment.generate();
    // 时间戳
    const payTimestamp = new Date().getTime();
    // 签名
    const data = `${appid}\n${payTimestamp}\n${payNonceStr}\nprepay_id=${prepay_id}\n`;
    const createSign = payment.rsaSign(data, privateKey);
    const host = this.ctx.header.referer.split('/')[2];
    const redirect_uri = `${this.ctx.protocol}://${host}/yl-web/payFulfil`;
    // TODO: 重定向到支付页面
    await this.ctx.render('pay.njk', { openid, appid, prepay_id, payNonceStr, payTimestamp, createSign, redirect_uri, out_trade_no });
  }
  // 关闭订单
  async orderClose({ out_trade_no }) {
    const { wxapi } = this.app.config;
    const { Order: model } = this.ctx.model;
    // 32位随机字符串
    const payment = new Payment({
      appid: wxapi.appid,
      mchid: wxapi.mchid,
      private_key: readFileSync(wxapi.wxKey).toString(),
      serial_no: wxapi.certid,
      apiv3_private_key: wxapi.v3key,
    });
    const res = await payment.close({ out_trade_no });
    if (res.status === 204) {
      // 订单关闭成功  删除原订单
      await model.findOne({ out_trade_no }).remove();
    } else {
      throw new BusinessError(ErrorCode.SERVICE_FAULT, '订单关闭失败');
    }
    return res;
  }
}

module.exports = WeixinAuthService;
